/* Program FILTER.C */
/* By Harald Stauss */
/* Last change 12.07.95 */

#include <ctype.h>
#include <math.h>
#include "nrc/nrutil.h"
#include "nrc/nrutil.c"
#include "nrc/four1.c"
#include "nrc/moment.c"
#include "tsa.h"

main(int argc, char *argv[])
{
  unsigned long     fn, n;
  int      i, ford;
  short    lh, add_mean;
  float    fc, sr, wg;
  float    *f,*h;
  float    ave, adev, sdev, var, skew, curt;
  char     *fin, *fout, *ftin, *ftout, *pftin, *pftout;
  REGIST   r;

  unsigned long     AnzFFT, segp;

  cout << "filter Ver.1.0 by H. Stauss, 1997 \n";

  /* D a t a a r r a y   I n i t i a l i z a t i o n */
  fin=new char[255]; 
  ftin=new char[255];
  fout=new char[255]; 
  ftout=new char[255];
  f=new float[1];
  h=new float[1];

  if (argc!=7 && argc!=8) {
    cout << "Number of parameters wrong! \n";
    cout << "Butterworth Filter \n";
    cout << "filter f_in f_out fc ord lh add_mean [sr] \n";
    cout << "f_in    : name of input file (*.TSA or *.ASC). \n";
    cout << "f_out   : name of output file (*.TSA or *.ASC). \n";
    cout << "fc      : corner frequency of the filter. \n";
    cout << "ord     : order of the filter. \n";
    cout << "lh      : 0 - low pass filter, 1 - high pass filter \n";
    cout << "add_mean: 0 - don't add mean, 1 - add mean \n";
    cout << "sr      : sampling rate (required when filetype ASCII). \n";
    goto ende;
  }

  /* P a r a m e t e r   e i n l e s e n */
  strcpy(fin, argv[1]); strcpy(ftin, argv[1]);
  strcpy(fout, argv[2]); strcpy(ftout, argv[2]);
  pftin =strrchr(ftin,'.');
  pftout=strrchr(ftout,'.');
  for (i=0; pftin[i]; i++) pftin[i]=toupper(pftin[i]);
  for (i=0; pftout[i]; i++) pftout[i]=toupper(pftout[i]);

  /* Abfrage nach Anzahl der Parameter */
  sscanf(argv[3],"%f",&fc);
  ford=atoi(argv[4]);
  lh=atoi(argv[5]);
  add_mean=atoi(argv[6]);
  if (strcmp(pftin,".ASC")==0) sscanf(argv[7],"%f",&sr);
  cout << "Input file           : " << fin << "\n";
  cout << "Output file          : " << fout << "\n";
  cout << "corner frequency     : " << fc << "\n";
  cout << "filter order         : " << ford << "\n";
  if (strcmp(pftin,".ASC")==0) cout << "sampling rate        : " << sr << "\n";

  /* E i n g a b e d a t e i   o e f f n e n */
  if (strcmp(pftin,".TSA")==0) r.ReadTSA(fin);
  if (strcmp(pftin,".ASC")==0) r.ReadASCII(fin,sr);
  if ((strcmp(pftin,".TSA")!=0) && (strcmp(pftin,".ASC")!=0)) {
    printf ("File type for input not recognized! \n");
    goto ende;
  }
  if (strcmp(pftin,".TSA")==0) cout << "sampling rate        : " << r.ch[0].sr << "\n";
  cout << "Number of channels   : " << r.n << "\n";

  /* M A I N L O O P  */
  for (i=0; i<r.n; i++) {
    if (floor(log(r.ch[i].n)/log(2))==log(r.ch[i].n)/log(2))
      fn=(unsigned long)pow(2,floor(log(r.ch[i].n)/log(2))); 
    else  
      fn=(unsigned long)pow(2,floor(log(r.ch[i].n)/log(2))+1); 
    cout << "n for channel " << i << ": " << fn << "\n";
    f=(float *)realloc(f,2*fn*sizeof(float));
    h=(float *)realloc(h,2*fn*sizeof(float));

    /* Filter definieren */
    wg=fc*fn/r.ch[i].sr;
    for (n=0; n<=fn/2;n ++) {
      if (lh==0) h[2*n]=sqrt(1/(1+pow(n/wg,2*ford)));
      else h[2*n]=sqrt(1/(1+pow(wg/(n+0.0001),2*ford)));
      h[2*n+1]=0;
    } /* endfor */
    for (n=1; n<fn/2; n++) {
      h[fn+n*2]=h[fn-n*2];
      h[fn+n*2+1]=0;
    } /* endfor */
 
    /* Zero padding */
    moment(r.ch[i].data-1,r.ch[i].n,&ave,&adev,&sdev,&var,&skew,&curt);

    /* Daten kopieren */
    for (n=0; n<r.ch[i].n; n++) {
      f[2*n]=r.ch[i].data[n]-ave;
      f[2*n+1]=0;
    } /* endfor */
    for (n=r.ch[i].n; n<fn; n++) {
      f[2*n]=0;
      f[2*n+1]=0;
    } /* endfor */

    /* filter */
    four1(f-1,fn,1);
    for (n=0; n<fn; n++) { 
      f[2*n]   = f[2*n]  * h[2*n];
      f[2*n+1] = f[2*n+1]* h[2*n];
    } /* endfor */  
    four1(f-1,fn,-1);

    /* gefilterte Daten kopieren */
    for (n=0; n<r.ch[i].n; n++) {
      if (add_mean) r.ch[i].data[n]=f[n*2]/fn+ave;
      else r.ch[i].data[n]=f[n*2]/fn;
    }

    /* Parameter fuer Kanaele setzen */
    for (n=0; n<r.ch[i].n; n++) {
      if (r.ch[i].data[n]<r.ch[i].min) r.ch[i].min=r.ch[i].data[n];
      if (r.ch[i].data[n]>r.ch[i].max) r.ch[i].max=r.ch[i].data[n];
    } /* endfor */
  } /* endfor Mainloop */

  /* D a t e i   s c h r e i b e n */
  if (strcmp(pftout,".TSA")==0) r.WriteTSA(fout);
  if (strcmp(pftout,".ASC")==0) r.WriteASCII(fout);
  if ((strcmp(pftout,".TSA")!=0) && (strcmp(pftout,".ASC")!=0)) {
    cout << "File type for output not recognized! \n";
    goto ende;
  }

ende:
  delete [] fin; delete [] ftin; delete [] fout; delete [] ftout;
  delete [] f; delete [] h;
  return(0);
} /* Ende von main */
